@@ -4,11 +4,13 @@
 # with support for scientific notation, and symbols for e and pi.
 # Extended to add exponentiation and simple built-in functions.
 # Extended test cases, simplified pushFirst method.
+# Removed unnecessary expr.suppress() call (thanks Nathaniel Peterson!), and added Group
+# Changed fnumber to use a Regex, which is now the preferred method
 #
-# Copyright 2003-2006 by Paul McGuire
+# Copyright 2003-2009 by Paul McGuire
 #
-from pyparsing import Literal,CaselessLiteral,Word,Combine,Group,Optional,\
-    ZeroOrMore,Forward,nums,alphas
+from pyparsing import Literal,CaselessLiteral,Word,Group,Optional,\
+    ZeroOrMore,Forward,nums,alphas,Regex
 import math
 import operator
 
@@ -38,9 +40,10 @@ def BNF():
     if not bnf:
         point = Literal( "." )
         e     = CaselessLiteral( "E" )
-        fnumber = Combine( Word( "+-"+nums, nums ) + 
-                           Optional( point + Optional( Word( nums ) ) ) +
-                           Optional( e + Word( "+-"+nums, nums ) ) )
+        #~ fnumber = Combine( Word( "+-"+nums, nums ) + 
+                           #~ Optional( point + Optional( Word( nums ) ) ) +
+                           #~ Optional( e + Word( "+-"+nums, nums ) ) )
+        fnumber = Regex(r"[+-]?\d+(:?\.\d*)?(:?[eE][+-]?\d+)?")
         ident = Word(alphas, alphas+nums+"_$")
      
         plus  = Literal( "+" )
@@ -55,7 +58,8 @@ def BNF():
         pi    = CaselessLiteral( "PI" )
         
         expr = Forward()
-        atom = (Optional("-") + ( pi | e | fnumber | ident + lpar + expr + rpar ).setParseAction( pushFirst ) | ( lpar + expr.suppress() + rpar )).setParseAction(pushUMinus) 
+        atom = (Optional("-") + ( pi | e | fnumber | ident + lpar + expr + rpar ).setParseAction( pushFirst ) | 
+                Group( lpar + expr + rpar )).setParseAction(pushUMinus) 
         
         # by defining exponentiation as "atom [ ^ factor ]..." instead of "atom [ ^ atom ]...", we get right-to-left exponents, instead of left-to-righ
         # that is, 2^3^2 = 2^(3^2), not (2^3)^2.
